/*
// Author: Neven Nikolov , neven1981@yahoo.com
// Aim:    Represents a NodeService unit
// To Do:  N / A
*/

package bg.unisofia.fmi.grosd.core.nodeservice;

import bg.unisofia.fmi.grosd.core.nodeservice.hardware.*;
import bg.unisofia.fmi.grosd.core.nodeservice.service.TaskStatus;
import bg.unisofia.fmi.grosd.core.monitoring.ejb.*;
import java.rmi.RemoteException;
import java.util.Vector;
import javax.ejb.*;

public class NS implements NSServiceIF
{
//members
	private: NodeHardware    m_Hardware;
	private: NodeTaskManager m_TaskManager;
	private  String          m_NodeID;
	
//Constructor
	public NS()
	{
		m_Hardware    = new NodeHardware( );
		m_TaskManager = new NodeTaskManager( );
		
		m_NodeID      = new String("unknown");
	}	
	
	public void MSNotifyAlive( )
	{	
		//Notify alive the Monitoring Service
		InitialContext ic = new InitialContext( );
		MonitoringServiceLocalHome lhhome = 
			(MonitoringServiceLocalHome) ic.lookup("LeaseHandlerLocal");
		MonitoringServiceLocal lh = lhhome.create( );
		lh.NotityAliveNS( m_NodeID );
	}
	
	public void SetNodeID( String a_NodeID )
	{
		m_NodeID = a_NodeID;
	}
	
	public String GetNodeID( )
	{
		return m_NodeID;
	}
//NSServiceIF implementation
    public boolean UpdateDynamicInfo( )
    {
    	return m_Hardware.UpdateDynamicInformation( );	
    }	
    
	public long GetAvailableMemoryRam( )
 	{
 		return m_Hardware.GetRam( ).GetAvailPhysical( );
  	}
	
	public long GetTotalMemoryRam( )
	{
		return m_Hardware.GetRam( ).GetTotalPhysical( );
	}
	
	public long GetCPUCount( )
	{
		return m_Hardware.GetCPU( ).GetCount( );
	}
		
	public long GetHardDiskTotalSpace( )
	{   //Size returned will be in GB
		return m_Hardware.GetHardStorage( ).GetSizeInMb( ) / 1000;
	}
	
	public TaskStatus GetStatusOfTask( int a_taskID )
 	{
		return m_TaskManager.getTaskStatus( a_taskID );
	}
	public boolean TerminateTask( int a_taskID)
 	{
		return m_TaskManager.terminateTask( a_taskID );
	}
		
	public Vector GetLoadedTasksID( )
 	{
		return m_TaskManager.GetLoadedTasksID( );
	}
	
	public boolean UploadTask(int a_taskID)
 	{
		return m_TaskManager.UploadTask( a_taskID );
	}
}
