/*
// Author: Neven Nikolov , neven1981@yahoo.com
// Aim:    Represents a NodeServiceInterface unit
// To Do:  N / A
*/
package bg.unisofia.fmi.grosd.core.nodeservice;

import bg.unisofia.fmi.grosd.core.nodeservice.service.TaskStatus;
import java.util.Vector;
import java.rmi.RemoteException;

public interface NSServiceIF 
{

	/**
	* <p> Returns true when dynamic data in the node is updated successfully
	* @return if the update of dynamic information is successful
	*/
	public boolean UpdateDynamicInfo( ) throws RemoteException;
	/**
	* <p> Returns the total available ram of any kind on the node
	*it is taken dynamically<p>
	* @return total available ram
	*/
	public long GetAvailableMemoryRam( ) throws RemoteException;
	
	/**
	* <p> Returns the total ram of any kind on the node
	*it is taken from the static information<p>
	* @return total ram
	*/
	public long GetTotalMemoryRam( ) throws RemoteException;
	
	/**
	* <p> Returns the total CPU count available the node<p>
	* @return total available ram
	*/
	public long GetCPUCount( ) throws RemoteException;
		
	/**
	* <p> Returns the total available HardDiskSpace of any kind on the node
	*It is taken from the static information<p>
	* @return total available ram
	*/	
	public long GetHardDiskTotalSpace( ) throws RemoteException;

	/**
	*<p> Returns if the upload of a job to a node is successful<p>
	*@param a_TaskID a <code>int</code> object which is the id of the job  
	*@return total is the upload successful or not
	*/	
	public boolean UploadTask( int a_TaskID ) throws RemoteException;
		
	/**
	*<p> Returns the status of a job, that has been uploaded on a node<p>
	*@param a_TaskID a <code>int</code> object which is the id of the job  
	*@return total is the upload successful or not
	*/		
	public TaskStatus GetStatusOfTask( int a_TaskID ) throws RemoteException;
	
	/**
	*<p> Returns if the operation of termination a job  on the node is successful<p>
	*@param a_TaskID a <code>int</code> object which is the id of the job  
	*@return total is the termination successful or not
	*/
	public boolean TerminateTask( int a_TaskID ) throws RemoteException;
		
	/**
	*<p> Returns a vector of the ids of the jobs that are loaded on the Node<p>
	*@return total is the termination successful or not
	*/
	public Vector GetLoadedTasksID( ) throws RemoteException;
}
