/*
// Author: Neven Nikolov , neven1981@yahoo.com
// Aim:    Represents a Node unit.This including all the valuable
//         software and hardware characteristic of the machine.
// To Do:  N / A
*/

package bg.unisofia.fmi.grosd.core.nodeservice.hardware;

import bg.unisofia.fmi.grosd.core.nodeservice.useful.CErrorLogger;
import bg.unisofia.fmi.grosd.core.nodeservice.hardware.*;
import bg.unisofia.fmi.grosd.core.nodeservice.service.*;
import java.util.Date;
import java.util.Vector;
import bg.unisofia.fmi.grosd.core.nodeservice.*;

//Has second thread that is starting the tasks and is looking at their status
class CID2Process
{
	int m_ID;          //id for the task
	Process m_Process; //process for the task
	
	CID2Process( int a_ID , Process a_Process )
	{
		m_ID      = a_ID;
		m_Process = a_Process;
	}
	
	public String GetStatus( )
	{
		if( m_Process != null )
		{
			return GetProcessStatus( m_Process );
		}
		else
		{
			return "process null ptr";
		}
	}
	
	private String GetProcessStatus( Process p1 )
	{
		if( p1 != null )
		{
			try
			{
				if( p1.exitValue() == 0 )
				{
					return "finished";
				}
				else
				{
					return "abort";
				}
			}
			catch( Exception ex )
			{
				CErrorLogger.AddLog( "NodeTaskManager,GetProcessStatus+" + p1.toString( ) , ex.toString( ) );
				return "running";
			}
		}
		else
		{
			return "abort";
		}
	}
	
	public int GetID( )
	{
		return m_ID;
	}
	
	public void terminate()
	{
		m_Process.destroy();
	}
};

public class NodeTaskManager 
{
//members
 	private NSTaskThread m_TaskThread;
 	
 	//Tasks deployed and waiting to be commited
    protected Vector m_VecTasks;
	
	//Tasks that are already running or execution stopped not too long
	protected Vector m_VecRunningTasks;
	             

//methods
	//Constructor
	public NodeTaskManager( )
	{
		//Tasks
		m_VecTasks         = new Vector( );
		m_VecRunningTasks  = new Vector( );

		m_TaskThread       = null;
 		Start();
	}
	
	public void Start()
	{
		Stop( );
		//Start the NS and the corresponding operations
		try
		{
			m_TaskThread = new NSTaskThread( this );
			m_TaskThread.start();
		}
		catch( Exception ex )
		{
			CErrorLogger.AddLog( "NodeTaskManager,Start", ex.toString( ) );
		}
	}	
	
	public void Stop()
	{
		try
		{
			if( m_TaskThread != null )
			{
				m_TaskThread.stop();
			}
		}
		catch( Exception ex )
		{
			CErrorLogger.AddLog( "NodeTaskManager,Stop", ex.toString( ) );
		}
	}

	public boolean AddTask( Task a_newTask )
	{
		//Tasks should be ordered by date to be easier to be ran when necessary 
		if( a_newTask != null )
		{
			m_VecTasks.add( a_newTask );
			return true;
		}
		else
		{
			return false;
		}
	}
	
	public TaskStatus getTaskStatus( int a_TaskID )
	{
		//If task is in the waiting
		for( int i = 0 ; i < m_VecTasks.size() ; i++ )
		{
			Task tmpTask = ( Task ) m_VecTasks.get( i );
			int tmpID    = tmpTask.GetID( );
			if( tmpID == a_TaskID )	
			{
				return new TaskStatus( a_TaskID , "waiting");	
			}
		}
	
		//Tasks that are already running or execution stopped not too long!
		for( int i = 0 ; i < m_VecRunningTasks.size() ; i++ )
		{
			CID2Process tmpID2Proc = ( CID2Process )m_VecRunningTasks.get( i );
			int tmpID = tmpID2Proc.GetID( );
			
			if( tmpID == a_TaskID )
			{
				return new TaskStatus( a_TaskID , tmpID2Proc.GetStatus( ) );
			}
		}
		
		//Not in either lists.Never runned or finished too early.
 		
		return new TaskStatus( a_TaskID , "finished" );	
	}
 	
	public void checkForTasksToRun()
	{    
	    //Function that checks for tasks that can( it is time ) be executed
 		
		//Trying to start tasks
		Date curDate = new Date();
		for( int i = 0 ; i < m_VecTasks.size() ; i++ )
		{
			Task tmpTask = ( Task ) m_VecTasks.get( i );
			Date date = tmpTask.m_StartDate;
			if( date.after( curDate ) == true )
			{
				//run task
				m_VecTasks.remove( i );
				i--;// the moment we remove element at its pos can be other element for execution
				runTask( tmpTask );
			}
		}
		
		//Trying to remove all tasks that have finish execution
		for( int i = 0 ; i < m_VecRunningTasks.size() ; i++ )
		{
			CID2Process tmpID2Proc = ( CID2Process )m_VecRunningTasks.get( i );
			if( tmpID2Proc.GetStatus() != "running")
			{	
				m_VecRunningTasks.remove( i );
				i--;
			}
		}
 				
	}
	
	private boolean runTask( Task a_taskToRun )
	{
 	
		//When it is time for the task to be started it is started
		//Task an instance of a service
		try
		{
			Process process  = Runtime.getRuntime().exec("notepad");
			m_VecRunningTasks.add( new CID2Process( a_taskToRun.GetID() , process ) );
			return true;
		}
		catch( Exception ex )
		{
			CErrorLogger.AddLog( "NodeTaskManager,runTask+" + a_taskToRun.toString( ) , ex.toString( ) );
			return false;
		}
 		
	}
	
	
	public boolean terminateTask( Task a_taskToTerminate )
	{
 		int idTask = a_taskToTerminate.GetID( );
		return terminateTask( idTask );
	}
	
	public boolean terminateTask( int a_IDTaskToTerminate )
	{
		//If task is in the waiting list - remove it!
		for( int i = 0 ; i < m_VecTasks.size() ; i++ )
		{
			Task tmpTask = ( Task ) m_VecTasks.get( i );
			int tmpID    = tmpTask.GetID( );
			
			if( tmpID == a_IDTaskToTerminate )	
			{
				m_VecTasks.remove( i );
				return true;	
			}
		}
	
		//Tasks that are already running or execution stopped not too long!
		for( int i = 0 ; i < m_VecRunningTasks.size() ; i++ )
		{
			CID2Process tmpID2Proc = ( CID2Process )m_VecRunningTasks.get( i );
			int tmpID = tmpID2Proc.GetID( );
			
			if( tmpID == a_IDTaskToTerminate )
			{
				tmpID2Proc.terminate();
				m_VecRunningTasks.remove( i );
				return true;
			}
		}
 		
		return false;
	}
 	
	public Vector GetTasksData()
	{
		
		Vector vecTaskStatus = new Vector();
 		//If task is in the waiting list - remove it!
		for( int i = 0 ; i < m_VecTasks.size() ; i++ )
		{
			Task tmpTask = ( Task ) m_VecTasks.get( i );
			int id       = tmpTask.GetID( );
			vecTaskStatus.add( new TaskStatus( id , "waiting"));
		}
	
		//Tasks that are already running or execution stopped not too long!
		for( int i = 0 ; i < m_VecRunningTasks.size() ; i++ )
		{
			CID2Process tmpID2Proc = ( CID2Process )m_VecRunningTasks.get( i );
			vecTaskStatus.add( new TaskStatus( tmpID2Proc.GetID() , tmpID2Proc.GetStatus() ));
		}
		
		return vecTaskStatus;
	}
 
 	public Vector GetLoadedTasksID( )
 	{
		return null;
	}
	
	public boolean UploadTask(int a_taskID)
 	{
		return false;
	}
}
