/*
// Copyright (c) 2002-2005 Universidade Federal de Campina Grande
// 
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
// 
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA 02111-1307 USA
*/


/*
// Author: Neven Nikolov , neven1981@yahoo.com
// Aim:    Represents a Zipping / unzipping utility
// To Do:  N / A
*/

/*
// Partially code was based on the SimpleZip class written by:
// @author Alisson Wilker - aandrade@dsc.ufcg.edu.br
// @author Guilherme Mauro Germoglio Barbosa - guiga@dsc.ufcg.edu.br
// @version 1.0 Created on 18/08/2004
// After all the aim is still the same:to zip and unzip files .
*/

package bg.unisofia.fmi.grosd.core.nodeservice;

import bg.unisofia.fmi.grosd.core.nodeservice.useful.CErrorLogger;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

public class Zipper {

	/** a buffer */
	private static final int BUFFER = 2048;
    
	/**
	 * Zips files.
	 * @param filesToZip Filepaths to zip.
	 * @param zipFileName The zip file path with respective name.
	 * @throws IOException Thrown if an IO exception occurs
	 */
    public static void zipIt(String[] filesToZip, String zipFileName)
			throws IOException {

		byte[] buffer = new byte[18024];

		try {

			ZipOutputStream out = new ZipOutputStream(new FileOutputStream(
					zipFileName));

			// iterate through the array of files, adding each to the zip file
			for (int i = 0; i < filesToZip.length; i++) {
				// Associate a file input stream for the current file
				FileInputStream in = new FileInputStream(filesToZip[i]);

				// Add ZIP entry to output stream.
				out.putNextEntry(new ZipEntry(filesToZip[i]));

				// Transfer bytes from the current file to the ZIP file
				int len;
				while ((len = in.read(buffer)) > 0) {
					out.write(buffer, 0, len);
				}

				// Close the current entry
				out.closeEntry();

				// Close the current file input stream
				in.close();

			}
			// Close the ZipOutPutStream
			out.close();
		}
		catch (IllegalArgumentException iae) 
		{
			CErrorLogger.AddLog("Zipper,zipit+"+ zipFileName , iae.toString( ) );
		}
		catch (FileNotFoundException fnfe) 
		{
			CErrorLogger.AddLog("Zipper,zipit+"+ zipFileName , fnfe.toString( ) );
		}
		catch (IOException ioe) 
		{
			CErrorLogger.AddLog("Zipper,zipit+"+ zipFileName , ioe.toString( ) );	
		}
	}

    /**
     * Unzips files if necessary creating parent directories
     * @param file The zip file to be extracted.
     * @param destinationDirectory The destination directory where extracted files will be placed
	 * @throws IOException Thrown if an IO exception occurs
     */
    public static void unzipIt(File file, String destinationDirectory) throws IOException {

		// Specify file to decompress
		String inFileName = file.getAbsolutePath();

		File sourceZipFile = new File(inFileName);
		File unzipDestinationDirectory = new File(destinationDirectory);
		if(unzipDestinationDirectory.exists() == false)
		{
			if(unzipDestinationDirectory.mkdir() )
			{
				System.out.print("Directory created");
			}
		}
			
		// Open Zip file for reading
		ZipFile zipFile = new ZipFile(sourceZipFile, ZipFile.OPEN_READ);

		// Create an enumeration of the entries in the zip file
		Enumeration zipFileEntries = zipFile.entries();

		// Process each entry
		while (zipFileEntries.hasMoreElements()) {
			// grab a zip file entry
			ZipEntry entry = (ZipEntry) zipFileEntries.nextElement();

			String currentEntry = new File(entry.getName()).getName();//entry.getName();

			File destFile = new File(unzipDestinationDirectory + File.separator
					+ currentEntry);

			// extract file if not a directory
			if (!entry.isDirectory()) {
				BufferedInputStream is = new BufferedInputStream(zipFile
						.getInputStream(entry));
				int currentByte;
				// establish buffer for writing file
				byte data[] = new byte[BUFFER];

				// write the current file to disk
				FileOutputStream fos = new FileOutputStream(destFile);
				BufferedOutputStream dest = new BufferedOutputStream(fos,
						BUFFER);

				// read and write until last byte is encountered
				while ((currentByte = is.read(data, 0, BUFFER)) != -1) {
					dest.write(data, 0, currentByte);
				}
				dest.flush();
				dest.close();
				is.close();
			}
		}
		zipFile.close();
	}
 }
