/*
// Author: Neven Nikolov , neven1981@yahoo.com
// Aim:    Represents a LanCard unit
// To Do:  N / A
*/

package bg.unisofia.fmi.grosd.core.nodeservice.hardware;

import bg.unisofia.fmi.grosd.core.nodeservice.useful.InterfaceForConstants;
import java.util.Vector;
import java.lang.Integer;

public class LanCard 
{
 //members
	private String m_strProviderName;//manufacturer of the Lan Card
	private String m_strLanCardName; //name attached to specific kind ot Card
	private String m_strMacAddress;  //physical MAC address of the LanCard 
	private Vector m_VecRateCapability; // 10 / 100 / 1000 Mb
 
	private CommunicationProperty m_CommunicationProperty;
//methods
	// Constructor
	public LanCard( String a_strProviderName , String a_strLanCardName ,
					String a_strMacAddress )
	{
		SetProviderName( a_strProviderName );
		SetLanCardName(  a_strLanCardName );
		SetMacAddress(   a_strMacAddress ); 
		
		m_VecRateCapability     = new Vector( );
		m_CommunicationProperty = new CommunicationProperty( );
	}
	
	public LanCard()
	{
		this( InterfaceForConstants.m_strNOT_DEFINED ,
			  InterfaceForConstants.m_strNOT_DEFINED ,
		      InterfaceForConstants.m_strNOT_DEFINED );
	}
			
	public boolean AddRateCapability( int a_newCapability )
	{
		if( a_newCapability < 1)
		{
			return false;	
		}
		
		if( m_VecRateCapability == null )
		{
			m_VecRateCapability = new Vector();
		}
		
		if( m_VecRateCapability != null )
		{
			boolean bAlreadyRateAdded = SupportsRateCapability( a_newCapability );
			if( bAlreadyRateAdded == false )
			{
				m_VecRateCapability.add( new Integer( a_newCapability ) );	
			}
			
			return true;
		}
		else
		{
			return false;
		}
	}
	
	public boolean SupportsRateCapability( int a_rateToTest )
	{
		if( m_VecRateCapability == null)
		{
			return false;
		}
		
		int count_diff_rates = m_VecRateCapability.size();
		for( int i = 0 ; i < count_diff_rates ; i++ )
		{
			Integer curRate = (Integer)m_VecRateCapability.get( i ); 
			if( curRate != null )
			{
				int rate = curRate.intValue(); 			
				if( rate == a_rateToTest )
				{
					return true;
				} 				
			}
	
		}
		
		return false;	
	}
	
	public boolean IsValidMacAddress( String a_strMacAddress )
	{
		/*
		Not sure what is the correct representation
		Windows:00-02-44-8F-8A-8B
		Linux:  0002448F8A8B
		So for now any kind of the 2 notations should be thought as valid
		*/
		
		a_strMacAddress  = a_strMacAddress.trim( );
		int cbMacAddress = a_strMacAddress.length();
		if( cbMacAddress == 17 )
		{
			return CheckIsValidWindowsMacAddress( a_strMacAddress );
		}
		else if( cbMacAddress == 12 )
		{
			return CheckIsValidLinuxMacAddress( a_strMacAddress );
		}
		
		return false;	
	}
	
	//Setters and Getters
	//ProviderName
	public boolean SetProviderName( String a_strProviderName )
	{	
		m_strProviderName = a_strProviderName;
		return true;
	}
	
	public String  GetProviderName()
	{
		return m_strProviderName;
	}
	
	//LanCardName
	public boolean SetLanCardName( String a_strLanCardName )
	{	
		m_strLanCardName = a_strLanCardName;
		return true;
	}
	
	public String  GetLanCardName()
	{
		return m_strLanCardName;
	}
	
	//MacAddress
	public boolean SetMacAddress( String a_strMacAddress )
	{
		if( IsValidMacAddress( a_strMacAddress ))
		{
			m_strMacAddress = a_strMacAddress;
			return true;
		}
		else
		{
			return false;
		}		
	}	
	
	public String GetMacAddress()
	{
		return m_strMacAddress;
	}
	
	//private methods
	private boolean IsHexadecimal( char ch )
	{
		if( ch >= '0' && ch <= '9' )
		{
			return true;
		}
		else if( ch >= 'a' && ch <= 'z' )
		{
			return true;
		}
		else if( ch >= 'A' && ch <= 'Z' )
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	private boolean CheckIsValidWindowsMacAddress( String a_strMacAddress )
	{
		//Windows:00-02-44-8F-8A-8B
		a_strMacAddress.toUpperCase();
		a_strMacAddress.trim();
		int cbMacAddress =  a_strMacAddress.length();
		if( cbMacAddress == 17 )
		{
			int i = 0;
			int j = 0;
			for(i = 0 ; i < 4 ; i++)
			{
				for( j = 0 ; j < 3 ; j++ )
				{
					char ch = a_strMacAddress.charAt( i * 3 + j );			
					
					if( j != 2 )
					{
						if( IsHexadecimal( ch ) != true )
						{
							return false;
						}
					}
					else
					{
						//the figures delimiter , in our case '-'
						if( ch != '-' )
						{
							return false;
						}
					}
				}
			}
			
			return true;
		}
		else
		{
			return false;
		}
	}
	
	private boolean CheckIsValidLinuxMacAddress( String a_strMacAddress )
	{
		//Linux:  0002448F8A8B
		a_strMacAddress.toUpperCase();
		a_strMacAddress.trim();
		
		int cbMacAddress =  a_strMacAddress.length();
		if( cbMacAddress == 12 )
		{
			int i = 0;
			int j = 0;
			for(i = 0 ; i < 4 ; i++)
			{
				for( j = 0 ; j < 3 ; j++ )
				{
					char ch = a_strMacAddress.charAt( i * 3 + j );			
					if( IsHexadecimal( ch ) != true )
					{
						return false;
					}				
				}
			}
			
			return true;
		}
		else
		{
			return false;
		}
	}
	
	public void SetCommunicationProperty( CommunicationProperty a_CommunicationProperty )
	{
		m_CommunicationProperty = a_CommunicationProperty;		
	}
	
	public CommunicationProperty GetCommunicationProperty( )
	{
		return m_CommunicationProperty;
	} 
}

