/*
// Author: Neven Nikolov , neven1981@yahoo.com
// Aim:    Represents a class that should supply dynamic information about the system 
//         at runtime - about hardware usage, os type, hardware architecture , based on 
//         C++ API 		 
*/

package bg.unisofia.fmi.grosd.core.nodeservice.hardware;

import bg.unisofia.fmi.grosd.core.nodeservice.useful.CFileFuncs;
import bg.unisofia.fmi.grosd.core.nodeservice.useful.CErrorLogger;
import java.util.StringTokenizer;

public class SysDynInfoCAPI{
	
	final int RAM_INFO       = 1;
	final int USER_INFO      = 2;
	final int PC_INFO        = 3;
	final int HARDDISK_INFO  = 4;
	final int DISKSPACE_INFO = 5;
	
	
	public static HardStorage m_LastHDInfo = new HardStorage( );
	public static RAM  m_LastRamInfo       = new RAM( );
 	
	public boolean AddInformation( StringTokenizer a_nextRowOfInformation )
	{
		int count_tokens = a_nextRowOfInformation.countTokens();
		if( count_tokens < 2 )
		{
			return false;
		}
		
		String nextToken = a_nextRowOfInformation.nextToken( );
		int group = Integer.parseInt( nextToken );
		switch ( group )
		{
			case RAM_INFO:
			{
				return AddInformationRAM( a_nextRowOfInformation );
			}
			case USER_INFO:
			{
				return AddInformationUSER( a_nextRowOfInformation );
			}
			case PC_INFO:
			{
				return AddInformationPC( a_nextRowOfInformation );
			}
			case HARDDISK_INFO:
			{
				return AddInformationHardDisks( a_nextRowOfInformation );
			}
			case DISKSPACE_INFO:
			{
				return AddInformationDiskSpace( a_nextRowOfInformation );
			}
			default:
			{
				return false;
			}
		}
 	}
	
	private boolean AddInformationRAM( StringTokenizer a_nextRowOfInformation )
	{
		final int count_tokens = 5;
		String[] pStringArr = new String[ count_tokens ]; 
		String[] pExpl      = {"Used %" , "Total Phys" , "Avail Phys" , "Total Virt" , "Avail Virt" };
		
		int i = 1;
		while ( a_nextRowOfInformation.hasMoreTokens( ) && i <= count_tokens )
		{
			pStringArr[ i - 1 ] = a_nextRowOfInformation.nextToken( );
			i++;
		}
		
		
		if( i == count_tokens + 1 )
		{
			//RAM ram = new RAM( );
			m_LastRamInfo.SetPercentUsed(   Integer.parseInt( pStringArr[ 0 ] ) );
			m_LastRamInfo.SetTotalPhysical( Integer.parseInt( pStringArr[ 1 ] ) );
			m_LastRamInfo.SetAvailPhysical( Integer.parseInt( pStringArr[ 2 ] ) );
			m_LastRamInfo.SetTotalVirtual(  Integer.parseInt( pStringArr[ 3 ] ) );
			m_LastRamInfo.SetAvailVirtual(  Integer.parseInt( pStringArr[ 4 ] ) );
			
			return false;
		}
		
		return true;
	}
	
	private boolean AddInformationUSER( StringTokenizer a_nextRowOfInformation )
	{
		final int count_tokens = 1;
		String[] pStringArr = new String[ count_tokens ]; 
		String[] pExpl      = {"User"};
		
		int i = 1;
		while ( a_nextRowOfInformation.hasMoreTokens( ) && i <= count_tokens )
		{
			
			pStringArr[ i - 1 ] = a_nextRowOfInformation.nextToken( );
			i++;
		}
		
		
		if( i == count_tokens + 1 )
		{
			for( i = 0 ; i < count_tokens ; i++ )
			{	
//				System.out.print( pExpl[ i ]);
//				System.out.print( pStringArr[ i ]);
			}
			
//			System.out.println("");
			return true;
		}
	
		return false;
	}

	private boolean AddInformationPC( StringTokenizer a_nextRowOfInformation )
	{
		final int count_tokens = 1;
		String[] pStringArr = new String[ count_tokens ]; 
		String[] pExpl      = {"PC User"};
		
		int i = 1;
		while ( a_nextRowOfInformation.hasMoreTokens( ) && i <= count_tokens )
		{
			
			pStringArr[ i - 1 ] = a_nextRowOfInformation.nextToken( );
			i++;
		}
		
		
		if( i == count_tokens + 1 )
		{
			for( i = 0 ; i < count_tokens ; i++ )
			{	
//				System.out.print( pExpl[ i ]);
//				System.out.print( pStringArr[ i ]);
			}
//			System.out.println("");
			return true;
		}
		
		return false;
	}		
	
	private boolean AddInformationHardDisks( StringTokenizer a_nextRowOfInformation )
	{
		final int count_tokens = 1;
		String[] pStringArr = new String[ count_tokens ]; 
		String[] pExpl      = {"GB Size:"};
		
		int i = 1;
		while ( a_nextRowOfInformation.hasMoreTokens( ) && i <= count_tokens )
		{
			
			pStringArr[ i - 1 ] = a_nextRowOfInformation.nextToken( );
			i++;
		}
		
		
		if( i == count_tokens + 1 )
		{
			//Populate Hard Disks Information when needed
		}
		
		return false;
	}	
	
	private boolean AddInformationDiskSpace( StringTokenizer a_nextRowOfInformation )
	{
		final int count_tokens = 3;
		String[] pStringArr = new String[ count_tokens ]; 
		String[] pExpl      = {"Partition","Used","Free"};
		
		int i = 1;
		while ( a_nextRowOfInformation.hasMoreTokens( ) && i <= count_tokens )
		{
			
			pStringArr[ i - 1 ] = a_nextRowOfInformation.nextToken( );
			i++;
		}
		
		
		if( i == count_tokens + 1 )
		{
			//m_LastHDInfo.SetSizeInMb( ) = Integer.parseInt( pStringArr * 1000 ); 	
			int addUsed  = Integer.parseInt( pStringArr[ 1 ] );
			int addFree  = Integer.parseInt( pStringArr[ 2 ] );
			int addTotal = addUsed + addFree;  
			
			int total = m_LastHDInfo.GetSizeInMb( ); 
			int used  = m_LastHDInfo.GetUsedSpaceInMb( );
			
			m_LastHDInfo.SetSizeInMb( total + addTotal );
			m_LastHDInfo.SetUsedSpaceInMb( used + addUsed );
			
			return true;
		}
		
		return false;
	}

	private native void UpdateInformationFile( String a_strDataFile );
	
	private void ResetContent( )
	{
		m_LastHDInfo.Reset( );
		m_LastRamInfo.Reset( );
	}	
	
	public boolean UpdateInformation( )
	{
		String tmpFile = "\\tmpfile.txt";
		tmpFile = CFileFuncs.GetProgramCacheDirectory( ) + tmpFile;
		
		System.out.println( tmpFile );
		try
		{
			//1.Reset old content
			ResetContent( );
			
			//2.Run the executable and fill the tmp file 
			UpdateInformationFile( tmpFile );	
			
			//3.Read the tmp file
			CFileFuncs.ReadFromDisk( tmpFile , this );
			return true;
		}
		catch( Exception ex )
		{
			CErrorLogger.AddLog( "SysDynInfoCAPI,UpdateInformation" , ex.toString( ) );
			return false;
		}
	}
		
	static 
	{
		//To load the DLL that calls my C++ code for system dynamic information
    	System.loadLibrary("updateOSInfo");
    }
}
