/*
// Author: Neven Nikolov , neven1981@yahoo.com
// Aim:    Represents a class that should supply dynamic information about the system 
//         at runtime - about hardware usage, os type, hardware architecture 		 
// To Do:  Should be added wrappers of C++ API functions to get more and more precise information 
//         about the system
*/

package bg.unisofia.fmi.grosd.core.nodeservice.hardware;

import bg.unisofia.fmi.grosd.core.nodeservice.useful.CErrorLogger;
import bg.unisofia.fmi.grosd.core.nodeservice.useful.*;
import java.lang.management.*;

public class SystemDynamicInfo 
{
  
	//CPU properties
	public static String GetCPUVendor()
	{
		String str  = new String("HARDWARE\\DESCRIPTION\\SYSTEM\\CENTRALPROCESSOR\\0");
		String str1 = new String("VendorIdentifier");
		
		String strResult = MyWinRegWrapper.GetValueString(str , str1 );
		String[] vendors = {"INTEL" , "AMD" , "NSC", "CYRIX" , "NEXGEN" , "IDT" , "UMC" , "RISE" , "TRANSMETA"};
		
		strResult = strResult.toUpperCase();
		for( int i = 0 ; i < vendors.length ; i++ )
		{
			if( CStringFuncs.ContainsCaseInsensitive(strResult , vendors[ i ] ) == true)
			{
				return vendors[ i ];
			}
		}
		
		return "UNKNOWN VENDOR";
	}
	
	public static int GetCPUMz()
	{
		String str  = new String("HARDWARE\\DESCRIPTION\\SYSTEM\\CENTRALPROCESSOR\\0");
		String str1 = new String("~Mhz");
		
		return MyWinRegWrapper.GetValueInt(str , str1 );
	}
	
	public static String GetCPUProcessorName()
	{
		String str  = new String("HARDWARE\\DESCRIPTION\\SYSTEM\\CENTRALPROCESSOR\\0");
		String str1 = new String("ProcessorNameString");
		
		return MyWinRegWrapper.GetValueString( str , str1 ).trim();
	}

	public static int GetCPUCount()
	{
		//OperatingSystemMxBean
		OperatingSystemMXBean runtime = ManagementFactory.getOperatingSystemMXBean();
		return runtime.getAvailableProcessors();		
	}
	
	//OS
	public static String GetNTServicePackFromRegistry()
	{
		String str  = new String("Software\\Microsoft\\Windows NT\\CurrentVersion");
		String str1 = new String("CSDVersion");
		
		return MyWinRegWrapper.GetValueString( str , str1 ).trim();
	}
	
	
	public static String GetPropertyVal( String a_strPropName )
	{
		try
		{
			return 	System.getProperty( a_strPropName );
		}
		catch( Exception ex )
		{
			CErrorLogger.AddLog( "SystemDynamicInfo,GetPropertyVal+" + a_strPropName , ex.toString() );
			return "";
		}
	}
	
	public static String GetJavaBinDirectory()
	{
		return GetJavaHome() + "\\..\\bin\\";
	}
	
	public static String GetUserDir()
	{
		return GetPropertyVal("user.dir");
	}
	
	public static String GetUserName()
	{
		return GetPropertyVal("user.name");
	}
	
	public static String GetUserHome()
	{
		return GetPropertyVal("user.home");
	}
	
	public static String GetOSVersion()
	{
		return GetPropertyVal("os.version");
	}
	
	public static String GetOSName()
	{
		return GetPropertyVal("os.name");
	}

	public static String GetOSArch()
	{
		return GetPropertyVal("os.arch");
	}
		
		
	public static String GetJavaVersion()
	{
		return GetPropertyVal("os.version");
	}
	
	public static String GetJavaCompiler()
	{
		return GetPropertyVal("java.compiler");
	}	

	public static String GetJavaVendor()
	{
		return GetPropertyVal("java.vendor");
	}

	public static String GetJavaVendorURL()
	{
		return GetPropertyVal("java.vendor.url");
	}		

	public static String GetJavaLibraryPath()
	{
		return GetPropertyVal("java.library.path");
	}
		
	public static String GetJavaClassVersion()
	{
		return GetPropertyVal("java.class.version");
	}
	
	public static String GetJavaClassPath()
	{
		return GetPropertyVal("java.class.path");
	}		

	public static String GetJavaHome()
	{
		return GetPropertyVal("java.home");
	}

	public static String GetJavaSpecifVendor()
	{
		return GetPropertyVal("java.specification.vendor");
	}		

	public static String GetJavaVMSpecifVendor()
	{
		return GetPropertyVal("java.vm.specification.vendor");
	}
		
	public static String GetJavaVMSpecifName()
	{
		return GetPropertyVal("java.vm.specification.name");
	}

	public static String GetJavaVMVersion()
	{
		return GetPropertyVal("java.vm.version");
	}
	
	public static String GetJavaVMVendor()
	{
		return GetPropertyVal("java.vm.vendor");
	}	

	public static String GetJavaVMName()
	{
		return GetPropertyVal("java.vm.name");
	}

	public static String GetJavaSpecifVersion()
	{
		return GetPropertyVal("java.specification.version");
	}
	
	public static String GetJavaSpecifName()
	{
		return GetPropertyVal("java.specification.name");
	}

	public static String GetJavaIOTmpDir()
	{
		return GetPropertyVal("java.io.tmpdir");
	}
	
	public static String GetJavaExtDirs()
	{
		return GetPropertyVal("java.ext.dirs");
	}
	
	public static String GetFileSeparator()
	{
		return GetPropertyVal("file.separator");
	}	
	
	public static String GetPathSeparator()
	{
		return GetPropertyVal("path.separator");
	}
	
	public static String GetLineSeparator()
	{
		return GetPropertyVal("line.separator");
	}
 }
