package bg.unisofia.fmi.grosd.core.nodeservice.rmi;

/*
// Author: Neven Nikolov , neven1981@yahoo.com
// Aim:    Represents a RMI object that will encapsulate communication with the node service
//         In fact it will call remote methods on the Node Service
// To Do:  N / A
*/

import java.rmi.*;
import java.rmi.server.*;
import java.util.Vector;
import bg.unisofia.fmi.grosd.core.nodeservice.useful.CErrorLogger;
import bg.unisofia.fmi.grosd.core.nodeservice.service.TaskStatus;

public class RMINSClient 
{
	//members
	private String m_strLastHostName;
	private int    m_Lastport;
	private RMINSInterface m_NSInterface;
	
	RMINSClient( )
	{
		m_NSInterface     = null;
		m_strLastHostName = "";
		m_Lastport        = 2000;	
	}
	 
	//Setter & Getters
	void SetHostPort( int a_port )
	{
		m_Lastport = a_port;
	}
	
	int GetHostPort( )
	{
		return m_Lastport;
	}
	
	void SetHostName( String a_strHostName )
	{
		m_strLastHostName = a_strHostName;
	}
	
	String GetHostName( )
	{
		return m_strLastHostName;
	} 
	
	public boolean IsAlive( String a_strHostName , int a_port )
	{
		return bind( a_strHostName , a_port );	
	}
	
	public boolean IsAlive( )
  	{
  		//rebind and call a method
  		return rebind( );
  	}
  	
	//RMINSInterface methods 
	public boolean rebind( )
	{
		return bind( m_strLastHostName , m_Lastport );
	} 
	
	public boolean bind( String a_strHostName , int a_port )
	{
		m_strLastHostName = a_strHostName;
		m_Lastport        = a_port;
		
		try
		{
			String strBinding = "rmi://" + a_strHostName + ":" + a_port + "/RMINS"; 
			m_NSInterface = ( RMINSInterface ) Naming.lookup ( strBinding );
		
			if( m_NSInterface != null )
			{	
				return true;
			}
			else
			{
				return false;
			}
        }
        catch( Exception ex )
        {
        	CErrorLogger.AddLog( "RMINSClient,bind+" + a_strHostName + "+" + a_port , ex.toString( ) );
        	return false;
        }
  	}
  	
  	public boolean UpdateDynamicInfo( )
  	{
  		if( m_NSInterface != null )
  		{
  			try
  			{
  				return m_NSInterface.UpdateDynamicInfo( ); 
  			}
  			catch( Exception ex )
  			{
  				CErrorLogger.AddLog( "RMINSClient,UpdateDynamicInfo"  , ex.toString( ) );
  				return false;
  			}
  		}
  		
  		return false;
  	}
  	
	public long GetAvailableMemoryRam( )
	{
		try
		{
			if( m_NSInterface != null )
	  		{
	  			return m_NSInterface.GetAvailableMemoryRam( );
	  		}
		}
		catch( Exception ex )
		{
 			CErrorLogger.AddLog( "RMINSClient,GetAvailableMemoryRam"  , ex.toString( ) );
 			return 0;
		}  	
  		
  		return 0;
	}
	
	public long GetTotalMemoryRam( )
	{
		try
		{
			if( m_NSInterface != null )
			{
				return m_NSInterface.GetTotalMemoryRam( );			
			}
		}
		catch( Exception ex )
		{
			CErrorLogger.AddLog( "RMINSClient,GetTotalMemoryRam"  , ex.toString( ) );
 			return 0;
		}
				
		return 0;
	}
	
	public long GetCPUCount( )
	{
		try
		{
			if( m_NSInterface != null )
			{
				return m_NSInterface.GetCPUCount( );
			}
		}
		catch( Exception ex )
		{
 			CErrorLogger.AddLog( "RMINSClient,GetCPUCount"  , ex.toString( ) );
 			return 0;
		}
				
		return 0;
	}
	
	public long GetHardDiskTotalSpace( )
	{
		try
		{
			if( m_NSInterface != null )
			{
				return m_NSInterface.GetHardDiskTotalSpace( );
			}
		}
		catch( Exception ex )
		{
 			CErrorLogger.AddLog( "RMINSClient,GetHardDiskTotalSpace"  , ex.toString( ) );
 			return 0;
		}		
		
		return 0;
	}
	
	public boolean UploadTask( int a_TaskID )
	{
		try
		{
			if( m_NSInterface != null )
			{
				return m_NSInterface.UploadTask( a_TaskID );
			}
		}
		catch( Exception ex )
		{
 			CErrorLogger.AddLog( "RMINSClient,UploadTask+" + a_TaskID  , ex.toString( ) );
 			return false;
		}		
		
		return false;
	}
	
	public TaskStatus GetStatusOfTask( int a_TaskID )
	{
		try
		{
			if( m_NSInterface != null )
			{
				return m_NSInterface.GetStatusOfTask( a_TaskID );
			}
		}
		catch( Exception ex )
		{
	 		CErrorLogger.AddLog( "RMINSClient,GetTaskStatus+" + a_TaskID  , ex.toString( ) );
 			return null;
		}		
		
		return null;		
	}
	
	public boolean TerminateTask( int a_TaskID )
	{
		try
		{
			if( m_NSInterface != null )
			{
				return m_NSInterface.TerminateTask( a_TaskID );
			}
		}
		catch( Exception ex )
		{
			CErrorLogger.AddLog( "RMINSClient,TerminateTask+" + a_TaskID  , ex.toString( ) );
 			return false;
		}			
		
		return false;
	}
	
	public Vector GetLoadedTasksID( )
	{
		try
		{
			if( m_NSInterface != null )
			{
				return m_NSInterface.GetLoadedTasksID( );
			}
		}
		catch( Exception ex )
		{
 			CErrorLogger.AddLog( "RMINSClient,GetLoadedTaskID"  , ex.toString( ) );
 			return new Vector( );
		}		
		
		return new Vector( );
	}
  	
  	public static void main( String[] args )
  	{
  		RMINSClient client = new RMINSClient( );
  		client.bind( "nikolov-99vm473" , 2000);
  		
  		client.UpdateDynamicInfo( );
  		System.out.print( client.GetAvailableMemoryRam( ) );
  	}
}
