/*
// Author: Neven Nikolov , neven1981@yahoo.com
// Aim:    Represents the RMS service interface
// To Do:  N / A
*/

package bg.unisofia.fmi.grosd.core.rmsservice;

import java.rmi.RemoteException;


/**
 * <p> This is the public interface(business logic) of the RMS Service. These methods correspond 
 * to all services that the RMS Service offers to the other systems services in
 * the grid. All methods throw <code>RemoteException</code> because this interface 
 * is extended by the remote interface of a session bean and all its methods have to 
 * throw such exceptions.
 *  
 * @author Neven Nikolov
 *
 */

interface RMSServiceIF {
	/**
	 * <p> Performs deployment of job to the RMS queue of waiting jobs. Upon succecc returns the id of the job in the RMS job queue,
       * which will be unique for that queue.<p>
	 * @param a_strDirectoryOfJobFiles a <code>String</code> object which contains 
	 * the directory that contains the user's job files - input , .exe(or java files) 
	 * @return the job's id for that queue
	 * @throws RemoteException
	 */
	public long deployJob(String a_strDirectoryOfJobFiles) 
		throws RemoteException;
	
	/**
	 * <p>Returns the available nodes for use in the cluster<p>
	 * @return the available nodes for use in the cluster
	 * @throws RemoteException
	 */
	public long getClusterAvailableNodes() 
		throws RemoteException;
			
	/**
	 * <p>Returns the available CPUs for use in the cluster<p>
	 * @return the available CPUs for use in the cluster
	 * @throws RemoteException
	 */
	public long getClusterAvailableCPUs() 
		throws RemoteException;
	
	/**
	 * <p> Returns the current position of the job in the RMS queue.<p>
	 * @param a_JobIDInQueue a <code>int</code>  
	 * @return the job's position in the queue
	 * @throws RemoteException
	 */
	public long getPositionOfJobInQueue(int a_JobIDInQueue) 
		throws RemoteException;
	
	
	/**
	 * <p> Returns the status of the job deployed to the RMS queue<p>
	 * @param a_JobIDInQueue a <code>int</code> object which is the id of the job 
	 * @return the status of the job
	 * @throws RemoteException
	 */

	public long getStatusOfJob(int a_JobIDInQueue) 
		throws RemoteException;
	
	/**
	 * <p> Terminates execution of job if it is running or just remove it from the RMS queue otherwise<p>
	 * @param a_JobIDInQueue a <code>int</code> object which is the id of the job 
	 * @return the operation status
	 * @throws RemoteException
	 */

	public boolean terminateJob(int a_JobIDInQueue) 
		throws RemoteException;	
	
	/**
	* <p> sets the scheduling algorithm/strategy for the RMS to arrange jobs / tasks in the scheduling queue<p>
	* @param a_strSchedulingAlgor a <code> String </code> object which is the id of the job 
	* @return the operation status
	* @throws RemoteException
	*/	
	public boolean setSchedulingAlgorithm( String a_strSchedulingAlgor )
		throws RemoteException;
	
	/**
	* <p> gets the scheduling algorithm/strategy for the RMS to arrange jobs / tasks in the scheduling queue<p>
	* @return the operation status
	* @throws RemoteException
	*/	
	public String getSchedulingAlgorithm( )
		throws RemoteException;
		
	/**
	* <p> sets the scheduling algorithm/strategy for the RMS to arrange jobs / tasks in the scheduling queue<p>
	* @return the operation status
	* @throws RemoteException
	*/
	//for now will be left dummy	
	public boolean scheduleJob(  )
		throws RemoteException;
}
